﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				AboutBox.cs
** description:	This file contains the “About Box” form.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Reflection;
using System.Windows.Forms;

namespace TX_Text_Control_Words
{
   partial class AboutBox : Form
   {
      public static new void Show()
      {
         (new AboutBox()).ShowDialog();
      }

      public static void Show(int nMilliseconds)
      {
         (new AboutBox(nMilliseconds)).ShowDialog();
      }

      private AboutBox()
      {
         InitializeComponent();
         Text = String.Format("About {0}", AssemblyProduct);
         _lblSubTitle.Text = (Is64BitAssembly() ? "64-bit" : "32-bit") + " Edition";
         _lblProductName.Text = ProductName;
         _lblVersion.Text = String.Format("Version {0}.{1}", AssemblyVersion.Major.ToString(), AssemblyVersion.Minor.ToString());
         _lblCopyright.Text = AssemblyCopyright;
      }

      private AboutBox(int nMilliseconds) : this()
      {
         _btnClose.Visible = false;
         _timer.Interval = nMilliseconds;
         _timer.Start();
      }

      #region Assembly Attribute Accessors

      public string AssemblyTitle
      {
         get
         {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
            if (attributes.Length > 0)
            {
               AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
               if (titleAttribute.Title != "")
               {
                  return titleAttribute.Title;
               }
            }
            return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
         }
      }

      public Version AssemblyVersion
      {
         get
         {
            return Assembly.GetExecutingAssembly().GetName().Version;
         }
      }

      public string AssemblyDescription
      {
         get
         {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false);
            if (attributes.Length == 0)
            {
               return "";
            }
            return ((AssemblyDescriptionAttribute)attributes[0]).Description;
         }
      }

      public string AssemblyProduct
      {
         get
         {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
            if (attributes.Length == 0)
            {
               return "";
            }
            return ((AssemblyProductAttribute)attributes[0]).Product;
         }
      }

      public string AssemblyCopyright
      {
         get
         {
            object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
            if (attributes.Length == 0)
            {
               return "";
            }
            return ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
         }
      }

      #endregion

      private void Timer_Tick(object sender, EventArgs e)
      {
         this.Close();
      }

      private void LinkLabel_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
      {
         System.Diagnostics.Process.Start("http://www.textcontrol.com/txtextcontrolwords/");
      }

      private bool Is64BitAssembly()
      {
         ImageFileMachine machine;
         PortableExecutableKinds peKind = 0;

         try
         {
            this.GetType().Module.GetPEKind(out peKind, out machine);
         }
         catch { }

         return ((peKind & System.Reflection.PortableExecutableKinds.PE32Plus) != 0);
      }
   }
}
