﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace Microsoft.Data.ConnectionUI
{
	public partial class DataConnectionDialog : Form
   {
      private class DataSourceCollection : ICollection<DataSource>
      {
         public DataSourceCollection(DataConnectionDialog dialog)
         {
            Debug.Assert(dialog != null);

            _dialog = dialog;
         }

         public int Count
         {
            get
            {
               return _list.Count;
            }
         }

         public bool IsReadOnly
         {
            get
            {
               return _dialog._showingDialog;
            }
         }

         public void Add(DataSource item)
         {
            if (item == null)
            {
               throw new ArgumentNullException("item");
            }
            if (_dialog._showingDialog)
            {
               throw new InvalidOperationException(Strings.DataConnectionDialog_CannotModifyState);
            }
            if (!_list.Contains(item))
            {
               _list.Add(item);
            }
         }

         public bool Contains(DataSource item)
         {
            return _list.Contains(item);
         }

         public bool Remove(DataSource item)
         {
            if (_dialog._showingDialog)
            {
               throw new InvalidOperationException(Strings.DataConnectionDialog_CannotModifyState);
            }
            bool result = _list.Remove(item);
            if (item == _dialog.SelectedDataSource)
            {
               _dialog.SetSelectedDataSource(null, true);
            }
            return result;
         }

         public void Clear()
         {
            if (_dialog._showingDialog)
            {
               throw new InvalidOperationException(Strings.DataConnectionDialog_CannotModifyState);
            }
            _list.Clear();
            _dialog.SetSelectedDataSource(null, true);
         }

         public void CopyTo(DataSource[] array, int arrayIndex)
         {
            _list.CopyTo(array, arrayIndex);
         }

         public IEnumerator<DataSource> GetEnumerator()
         {
            return _list.GetEnumerator();
         }

         System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
         {
            return _list.GetEnumerator();
         }

         private List<DataSource> _list = new List<DataSource>();
         private DataConnectionDialog _dialog;
      }
   }
}
