﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				DbConnectionAdapter.cs
** description:	This file contains an adapter class wrapping a DbConnection object.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Adapter class for DbConnection objects.
   /// </summary>
   internal abstract class DbConnectionAdapter
   {
      protected DataTableAdapter _selectedTable;
      protected List<string> _tableNames;
      protected List<string> _viewNames;

      internal DataTableAdapter SelectedTable {
         get { return _selectedTable; }
      }

      internal List<string> TableNames { 
         get { return _tableNames; }
      }

      internal List<string> ViewNames { 
         get { return _viewNames; }
      }

      internal abstract List<DataRelation> DataRelations { get; }

      internal abstract DataTable GetTable(string strTable);
      internal abstract void SelectTable(string strTableName);
      internal abstract void Close();
      internal abstract void GetTableAndViewNames();

      /// <summary>
      /// Returns a list of tables from the database.
      /// </summary>
      /// <param name="tableNames">Table names.</param>
      /// <returns>Tables.</returns>
      internal abstract DataSet GetDataSet(List<string> tableNames);

      /// <summary>
      /// Adds a data relation to the currently selected database and table.
      /// </summary>
      /// <param name="mainTableName">Main table name.</param>
      /// <param name="mainColName">Main table column name.</param>
      /// <param name="childTableName">Child table name.</param>
      /// <param name="childColName">Child table column name.</param>
      internal abstract bool AddDataRelation(string mainTableName, string mainColName, string childTableName, string childColName);
   } // abstract class DBConnectionAdapter
}
