﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				DeleteBookmarkDialog.cs
** description:	This file contains a dialog showing a list of bookmarks in a document.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using TXTextControl;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Dialog showing a list of bookmarks and buttons to navigate to 
   /// and remove bookmarks from that list.
   /// </summary>
   public partial class DeleteBookmarkDialog : Form
   {
      TXTextControl.TextControl _textControl;

      public const string MsgBookmkPartOfBlock = "Bookmark seems to be part of a mail merge block. Delete anyway?";

      public DeleteBookmarkDialog(List<DocumentTarget> bookmarks, TXTextControl.TextControl textControl)
      {
         InitializeComponent();

         _textControl = textControl;
         FillListBox(bookmarks);
      }

      private void FillListBox(List<DocumentTarget> bookmarks)
      {
         _lbBookmarks.DisplayMember = "TargetName";
         bookmarks.ForEach(block => _lbBookmarks.Items.Add(block));
      }

      private void lbBookmarks_SelectedIndexChanged(object sender, EventArgs e)
      {
         _btnDelete.Enabled = _btnGoto.Enabled = (_lbBookmarks.SelectedIndex > -1);
      }

      private void BtnGoto_Click(object sender, EventArgs e)
      {
         var selectedItem = _lbBookmarks.SelectedItem as DocumentTarget;
         if (selectedItem == null) return;

         selectedItem.ScrollTo();
      }

      private void BtnDelete_Click(object sender, EventArgs e)
      {
         var dlgRes = System.Windows.Forms.DialogResult.Yes;

         var selectedItem = _lbBookmarks.SelectedItem as DocumentTarget;
         if (selectedItem == null) return;

         if (selectedItem.TargetName.StartsWith(MergeBlock.BlockStartPrefix, StringComparison.OrdinalIgnoreCase)
            || selectedItem.TargetName.StartsWith(MergeBlock.BlockEndPrefix, StringComparison.OrdinalIgnoreCase))
         {
            // Ask user if a block marker may be deleted
            dlgRes = MessageBox.Show(
               MsgBookmkPartOfBlock,
               ProductName,
               MessageBoxButtons.YesNo,
               MessageBoxIcon.Question
            );
         }

         if (dlgRes != System.Windows.Forms.DialogResult.Yes) return;

         _textControl.DocumentTargets.Remove(selectedItem);
         _lbBookmarks.Items.Remove(selectedItem);
      }
   }
}
