﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				DeleteMergeBlockDialog.cs
** description:	This file contains a dialog showing a list of merge blocks in a document.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using TXTextControl;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Dialog showing a list of merge blocks and buttons to navigate to 
   /// and remove merge blocks from that list.
   /// </summary>
   public partial class DeleteMergeBlockDialog : Form
   {
      TextControl _textControl;

      public DeleteMergeBlockDialog(List<MergeBlock> blocks, TextControl textControl)
      {
         InitializeComponent();

         _textControl = textControl;
         FillListBox(blocks);
      }

      private void FillListBox(List<MergeBlock> blocks)
      {
         _lbBlockNames.DisplayMember = "Name";
         blocks.ForEach(block => _lbBlockNames.Items.Add(block));
      }

      private void lbBlockNames_SelectedIndexChanged(object sender, EventArgs e)
      {
         _btnDelete.Enabled = _btnGoto.Enabled = (_lbBlockNames.SelectedIndex > -1);
      }

      private void BtnGoto_Click(object sender, EventArgs e)
      {
         var selectedItem = _lbBlockNames.SelectedItem as MergeBlock;
         if (selectedItem == null) return;

         selectedItem.StartMarker.ScrollTo();
      }

      private void BtnDelete_Click(object sender, EventArgs e)
      {
         var selectedItem = _lbBlockNames.SelectedItem as MergeBlock;
         if (selectedItem == null) return;

         _textControl.DocumentTargets.Remove(selectedItem.StartMarker);
         _textControl.DocumentTargets.Remove(selectedItem.EndMarker);

         _lbBlockNames.Items.Remove(selectedItem);
      }
   }
}
