﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				FrmGoTo.cs
** description:	This file contains a small dialog which lets the user enter a location in a document.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.ComponentModel;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// A small dialog which lets the user enter a location in a document.
   /// </summary>
   public partial class FrmGoTo : Form
   {
      public FrmGoTo()
      {
         InitializeComponent();
         MaxPages = 1;
         MaxSections = 1;
         Number = 1;

         _lstPage.SelectedIndex = 0;
      }
      
      
      public int Number { get; private set; }
      public bool IsSection { get; private set; }
      public int MaxPages { get; set; }
      public int MaxSections { get; set; }


      private void FrmGoTo_Load(object sender, EventArgs e)
      {
         _lstPage.Focus();
      }

      private void LstPage_SelectedIndexChanged(object sender, EventArgs e)
      {
         switch (_lstPage.SelectedIndex)
         {
            case 0:
               IsSection = false;
               _lblWhat.Text = "Page:";
               break;

            case 1:
               IsSection = true;
               _lblWhat.Text = "Section:";
               break;
         }
      }

      private void BtnGo_Click(object sender, EventArgs e)
      {
         DialogResult = System.Windows.Forms.DialogResult.OK;
         Close();
      }

      private bool OnlyNumericChars(string str)
      {
         foreach (char c in str)
         {
            if (!Char.IsNumber(c)) return false;
         }

         return true;
      }

      private string CheckValue(int max, string text)
      {
         if (text == null) return "";
         if (text == "") return "";

         int value = 1;

         try
         {
            value = Math.Min(max, Convert.ToInt32(text));
         }
         catch { }

         return value.ToString();
      }

      private void TxtNumber_Validating(object sender, CancelEventArgs e)
      {
         e.Cancel = !OnlyNumericChars(_txtNumber.Text);
      }

      private void TxtNumber_TextChanged(object sender, EventArgs e)
      {
         _txtNumber.TextChanged -= TxtNumber_TextChanged;
         _txtNumber.Text = IsSection ? CheckValue(MaxSections, _txtNumber.Text) : CheckValue(MaxPages, _txtNumber.Text);
         try
         {
            Number = Convert.ToInt32(_txtNumber.Text);
            _btnGo.Enabled = true;
         }
         catch (FormatException) { _btnGo.Enabled = false; }
         _txtNumber.TextChanged += TxtNumber_TextChanged;
      }

      private void _txtNumber_Leave(object sender, EventArgs e)
      {
         if (_txtNumber.Text != "") return;
         _txtNumber.TextChanged -= TxtNumber_TextChanged;
         _txtNumber.Text = "1";
         Number = 1;
         _txtNumber.TextChanged += TxtNumber_TextChanged;
      }
   }
}
