﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				FrmPasteSpecial.cs
** description:	This file contains the “paste special” dialog.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// The “paste special” dialog.
   /// </summary>
   public partial class FrmPasteSpecial : Form
   {
      public TXTextControl.TextControl TextControl { get; set; }
      public object SelectedItem { get; private set; }

      public FrmPasteSpecial()
      {
         InitializeComponent();
         TextControl = null;
      }

      private void FrmPasteSpecial_Load(object sender, EventArgs e)
      {
         FillListBox();
      }

      private void FillListBox()
      {
         foreach (TXTextControl.ClipboardFormat cf in TextControl.GetClipboardFormats())
         {
            _lbPasteAs.Items.Add(cf);
         }
         _lbPasteAs.SelectedItem = SelectedItem = _lbPasteAs.Items[0];
      }

      private string GetClipboardFormatDescription()
      {
         string result = "";

         switch (SelectedItem.ToString())
         {
            case "HTMLFormat":
               result = "Specifies HTML format (Hypertext Markup Language).";
               break;

            case "Image":
               result = "Specifies an image.";
               break;

            case "PlainText":
               result = "Specifies unformatted text.";
               break;

            case "RichTextFormat":
               result = "Specifies RTF format (Rich Text Format).";
               break;

            case "TXTextControlFormat":
               result = "Specifies formatted text in the internal TX Text Control format.";
               break;

            case "TXTextControlImage":
               result = "Specifies an image including information about how the image is \r\npositioned in the document (inline or geometrically positioned).";
               break;

            case "TXTextControlTextframe":
               result = "Specifies a TX Text Control textframe including its contents. The \r\nclipbord also contains information about how the textframe is \r\npositioned in the document (inline or geometrically positioned).";
               break;

            default:
               result = "-";
               break;
         }

         return result;
      }

      private void LbPasteAs_SelectedIndexChanged(object sender, EventArgs e)
      {
         SelectedItem = _lbPasteAs.SelectedItem;
         _lblDescription.Text = GetClipboardFormatDescription();
      }

      private void _btnOK_Click(object sender, EventArgs e)
      {
         DialogResult = System.Windows.Forms.DialogResult.OK;
         Close();
      }
   }
}
