﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				InsertMergeBlockDialog.cs
** description:	This file contains the merge block insertion dialog.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// The merge block insertion dialog. Shows a list of the columns of the currently 
   /// selected data table and lets the user choose to create the merge block as a 
   /// table or a plain paragraph.
   /// </summary>
   public partial class InsertMergeBlockDialog : Form
   {
      #region "  Nested Types  "

      /// <summary>
      /// The type of block template to create (a table row or a plain paragraph)
      /// </summary>
      public enum BlockTemplateType
      {
         TableRow,
         PlainParagraph,
      };

      #endregion // Nested Types

      
      #region "  Fields  "

      TXTextControl.TextControl _textControl;
      DbConnectionAdapter _dbCon;

      #endregion // Fields


      #region "  Constructors  "

      internal InsertMergeBlockDialog(string strBlockName, DbConnectionAdapter dbCon, TXTextControl.TextControl textControl)
      {
         InitializeComponent();

         SelectedColumnNames = new List<string>();
         _textControl = textControl;
         _dbCon = dbCon;

         // Get table and fill list box with column names:

         DataTable table = _dbCon.GetTable(strBlockName);
         if (table == null) return;

         foreach (DataColumn col in table.Columns)
         {
            _checkedListBox.Items.Add(col.ColumnName);
         }
      }

      #endregion // Constructors


      #region "  Properties  "

      public List<string> SelectedColumnNames { get; private set; }

      public BlockTemplateType BlockType { get; private set; }

      #endregion // properties


      #region "  Event Handlers  "

      private void BtnOK_Click(object sender, EventArgs e)
      {
         if (_rbParagraph.Checked) BlockType = BlockTemplateType.PlainParagraph;
         else if (_rbTableRow.Checked) BlockType = BlockTemplateType.TableRow;

         foreach (string strTblCol in _checkedListBox.CheckedItems) SelectedColumnNames.Add(strTblCol);

         DialogResult = System.Windows.Forms.DialogResult.OK;
         Close();
      }

      private void CheckedListBox_SelectedIndexChanged(object sender, EventArgs e)
      {
         _btnOK.Enabled = _checkedListBox.CheckedItems.Count != 0;
      }

      #endregion // Event Handlers
   }
}
