﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				TabGrp_TableTools.cs
** description:	This file contains event handlers, fields and helper methods concerning the “Table Tools” 
**                contextual ribbon tab group.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Windows.Forms;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private Fields  "

      private RibbonButton _btnTblSelectCell;
      private RibbonButton _btnTblSelectColumn;
      private RibbonButton _btnTblSelectRow;
      private RibbonButton _btnTblSelectTable;
      private RibbonToggleButton _tglBtnTblViewGridlines;
      private RibbonButton _btnTblProperties;
      private RibbonButton _btnDeleteCells;
      private RibbonButton _btnDeleteCellsMoveLeft;
      private RibbonButton _btnDeleteCellsWholeRow;
      private RibbonButton _btnDeleteCellsWholeColumn;
      private RibbonButton _btnTblDeleteColumns;
      private RibbonButton _btnTblDeleteRows;
      private RibbonButton _btnTblDeleteTable;
      private RibbonButton _btnTblInsertAbove;
      private RibbonButton _btnTblInsertBelow;
      private RibbonButton _btnTblInsertLeft;
      private RibbonButton _btnTblInsertRight;
      private RibbonButton _btnTblMergeCells;
      private RibbonButton _btnTblSplitCells;
      private RibbonButton _btnTblSplitAbove;
      private RibbonButton _btnTblSplitBelow;

      private RibbonToggleButton[] _tglBtnTblCellAlign;

      #endregion // Private fields

      #region "  Enumerations  "

      enum RibbonMarkupCommand_TabGrpTableTools : uint
      {
         cmdTabGrpTableTools = 6000,
         cmdTabTableLayout = 6001,
         cmdGrpTblLayoutTable = 6002,
         cmdGrpTblLayoutRowsCols = 6003,
         cmdGrpTblLayoutMerge = 6004,
         cmdGrpTblLayoutAlignment = 6005,
         cmdDrpDnTblSelect = 6006,
         cmdBtnTblSelectCell = 6007,
         cmdBtnTblSelectColumn = 6008,
         cmdBtnTblSelectRow = 6009,
         cmdBtnTblSelectTable = 6010,
         cmdTglBtnTblViewGridlines = 6011,
         cmdBtnTblProperties = 6012,
         cmdDrpDnDelete = 6013,
         cmdBtnDeleteCells = 6014,
         cmdBtnDeleteCellsMoveToLeft = 6015,
         cmdBtnDeleteCellsWholeRow = 6016,
         cmdBtnDeleteCellsWholeColumn = 6018,
         cmdBtnTblDeleteColumns = 6019,
         cmdBtnTblDeleteRows = 6020,
         cmdBtnTblDeleteTable = 6021,
         cmdBtnTblInsertAbove = 6022,
         cmdBtnTblInsertBelow = 6023,
         cmdBtnTblInsertLeft = 6024,
         cmdBtnTblInsertRight = 6025,
         cmdBtnTblMergeCells = 6026,
         cmdBtnTblSplitCells = 6027,
         cmdDrpDnSplitTable = 6028,
         cmdBtnTblSplitAbove = 6029,
         cmdBtnTblSplitBelow = 6030,
         cmdTglBtnTblCellAlignTopLeft = 6031,
         cmdTglBtnTblCellAlignTopCenter = 6032,
         cmdTglBtnTblCellAlignTopRight = 6033,
         cmdTglBtnTblCellAlignTopJust = 6034,
         cmdTglBtnTblCellAlignMidLeft = 6035,
         cmdTglBtnTblCellAlignMidCenter = 6036,
         cmdTglBtnTblCellAlignMidRight = 6037,
         cmdTglBtnTblCellAlignMidJust = 6038,
         cmdTglBtnTblCellAlignBotLeft = 6039,
         cmdTglBtnTblCellAlignBotCenter = 6040,
         cmdTglBtnTblCellAlignBotRight = 6041,
         cmdTglBtnTblCellAlignBotJust = 6042,
      }

      enum TblCellAlignButton
      {
         TopLeft,
         TopCenter,
         TopRight,
         TopJust,
         MidLeft,
         MidCenter,
         MidRight,
         MidJust,
         BotLeft,
         BotCenter,
         BotRight,
         BotJust,
      }

      #endregion // Enumerations

      #region "  Ribbon event handlers  "

      void TglBtnTblCellAlign_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonToggleButton btn = sender as RibbonToggleButton;
         if (btn == null) return;

         var cmd = (RibbonMarkupCommand_TabGrpTableTools)btn.CommandID;

         if (btn.BooleanValue)
         {
            switch (cmd)
            {
               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopLeft:
                  TblCellAlignBtnPressed(TblCellAlignButton.TopLeft);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopCenter:
                  TblCellAlignBtnPressed(TblCellAlignButton.TopCenter);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopRight:
                  TblCellAlignBtnPressed(TblCellAlignButton.TopRight);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopJust:
                  TblCellAlignBtnPressed(TblCellAlignButton.TopJust);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidLeft:
                  TblCellAlignBtnPressed(TblCellAlignButton.MidLeft);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidCenter:
                  TblCellAlignBtnPressed(TblCellAlignButton.MidCenter);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidRight:
                  TblCellAlignBtnPressed(TblCellAlignButton.MidRight);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidJust:
                  TblCellAlignBtnPressed(TblCellAlignButton.MidJust);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotLeft:
                  TblCellAlignBtnPressed(TblCellAlignButton.BotLeft);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotCenter:
                  TblCellAlignBtnPressed(TblCellAlignButton.BotCenter);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotRight:
                  TblCellAlignBtnPressed(TblCellAlignButton.BotRight);
                  break;

               case RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotJust:
                  TblCellAlignBtnPressed(TblCellAlignButton.BotJust);
                  break;
            }
         }
         else
         {
            btn.BooleanValue = true;
         }
      }

      void BtnTblSplitBelow_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Split(TXTextControl.TableAddPosition.After);
      }

      void BtnTblSplitAbove_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Split(TXTextControl.TableAddPosition.Before);
      }

      void BtnTblSplitCells_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().SplitCells();
      }

      void BtnTblMergeCells_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().MergeCells();
      }

      void BtnTblInsertRight_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Columns.Add(TXTextControl.TableAddPosition.After);
      }

      void BtnTblInsertLeft_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Columns.Add(TXTextControl.TableAddPosition.Before);
      }

      void BtnTblInsertBelow_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Rows.Add(TXTextControl.TableAddPosition.After, 1);
      }

      void BtnTblInsertAbove_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Rows.Add(TXTextControl.TableAddPosition.Before, 1);
      }

      void BtnTblDeleteTable_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.Remove();
      }

      void BtnTblDeleteRows_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Rows.Remove();
      }

      void BtnTblDeleteColumns_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Columns.Remove();
      }

      void BtnDeleteCellsWholeColumn_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Columns.Remove();
      }

      void BtnDeleteCellsWholeRow_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Rows.Remove();
      }

      void BtnDeleteCellsMoveLeft_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Cells.Remove();
      }

      void BtnTblProperties_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.TableFormatDialog();
      }

      void TglBtnTblViewGridlines_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GridLines = _tglBtnTblViewGridlines.BooleanValue;
      }

      void BtnTblSelectTable_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Tables.GetItem().Select();
      }

      void BtnTblSelectRow_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _textControl.Tables.GetItem().Rows.GetItem().Select();
         }
         catch { }
      }

      void BtnTblSelectColumn_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _textControl.Tables.GetItem().Columns.GetItem().Select();
         }
         catch { }
      }

      void BtnTblSelectCell_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _textControl.Tables.GetItem().Cells.GetItem().Select();
         }
         catch { }
      }

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void CreateRibbonItems_TabGrpTableTools()
      {
         _btnTblSelectCell = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSelectCell);
         _btnTblSelectColumn = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSelectColumn);
         _btnTblSelectRow = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSelectRow);
         _btnTblSelectTable = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSelectTable);
         _tglBtnTblViewGridlines = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblViewGridlines);
         _btnTblProperties = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblProperties);
         _btnDeleteCells = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnDeleteCells);
         _btnDeleteCellsMoveLeft = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnDeleteCellsMoveToLeft);
         _btnDeleteCellsWholeRow = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnDeleteCellsWholeRow);
         _btnDeleteCellsWholeColumn = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnDeleteCellsWholeColumn);
         _btnTblDeleteColumns = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblDeleteColumns);
         _btnTblDeleteRows = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblDeleteRows);
         _btnTblDeleteTable = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblDeleteTable);
         _btnTblInsertAbove = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblInsertAbove);
         _btnTblInsertBelow = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblInsertBelow);
         _btnTblInsertLeft = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblInsertLeft);
         _btnTblInsertRight = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblInsertRight);
         _btnTblMergeCells = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblMergeCells);
         _btnTblSplitCells = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSplitCells);
         _btnTblSplitAbove = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSplitAbove);
         _btnTblSplitBelow = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdBtnTblSplitBelow);

         _tglBtnTblCellAlign = new RibbonToggleButton[Enum.GetValues(typeof(TblCellAlignButton)).Length];

         _tglBtnTblCellAlign[(int)TblCellAlignButton.TopLeft] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopLeft);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.TopCenter] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopCenter);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.TopRight] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopRight);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.TopJust] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignTopJust);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.MidLeft] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidLeft);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.MidCenter] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidCenter);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.MidRight] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidRight);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.MidJust] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignMidJust);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.BotLeft] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotLeft);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.BotCenter] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotCenter);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.BotRight] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotRight);
         _tglBtnTblCellAlign[(int)TblCellAlignButton.BotJust] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpTableTools.cmdTglBtnTblCellAlignBotJust);
      }

      private void AttachRibbonEvents_TabGrpTableTools()
      {
         _btnTblSelectCell.ExecuteEvent += BtnTblSelectCell_ExecuteEvent;
         _btnTblSelectColumn.ExecuteEvent += BtnTblSelectColumn_ExecuteEvent;
         _btnTblSelectRow.ExecuteEvent += BtnTblSelectRow_ExecuteEvent;
         _btnTblSelectTable.ExecuteEvent += BtnTblSelectTable_ExecuteEvent;
         _tglBtnTblViewGridlines.ExecuteEvent += TglBtnTblViewGridlines_ExecuteEvent;
         _btnTblProperties.ExecuteEvent += BtnTblProperties_ExecuteEvent;
         _btnDeleteCells.ExecuteEvent += BtnDeleteCellsMoveLeft_ExecuteEvent;
         _btnDeleteCellsMoveLeft.ExecuteEvent += BtnDeleteCellsMoveLeft_ExecuteEvent;
         _btnDeleteCellsWholeRow.ExecuteEvent += BtnDeleteCellsWholeRow_ExecuteEvent;
         _btnDeleteCellsWholeColumn.ExecuteEvent += BtnDeleteCellsWholeColumn_ExecuteEvent;
         _btnTblDeleteColumns.ExecuteEvent += BtnTblDeleteColumns_ExecuteEvent;
         _btnTblDeleteRows.ExecuteEvent += BtnTblDeleteRows_ExecuteEvent;
         _btnTblDeleteTable.ExecuteEvent += BtnTblDeleteTable_ExecuteEvent;
         _btnTblInsertAbove.ExecuteEvent += BtnTblInsertAbove_ExecuteEvent;
         _btnTblInsertBelow.ExecuteEvent += BtnTblInsertBelow_ExecuteEvent;
         _btnTblInsertLeft.ExecuteEvent += BtnTblInsertLeft_ExecuteEvent;
         _btnTblInsertRight.ExecuteEvent += BtnTblInsertRight_ExecuteEvent;
         _btnTblMergeCells.ExecuteEvent += BtnTblMergeCells_ExecuteEvent;
         _btnTblSplitCells.ExecuteEvent += BtnTblSplitCells_ExecuteEvent;
         _btnTblSplitAbove.ExecuteEvent += BtnTblSplitAbove_ExecuteEvent;
         _btnTblSplitBelow.ExecuteEvent += BtnTblSplitBelow_ExecuteEvent;

         foreach (var btn in _tglBtnTblCellAlign) btn.ExecuteEvent += TglBtnTblCellAlign_ExecuteEvent;
      }

      private void UpdateContextualTableTabGroup()
      {
         TXTextControl.Table tbl = _textControl.Tables.GetItem();
         bool bIsInTable = (tbl != null);

         _tabGroupTableTools.ContextAvailable = bIsInTable ? ContextAvailability.Available : ContextAvailability.NotAvailable;

         if (bIsInTable) UpdateTableLayoutTab(tbl);
      }

      private void UpdateTableLayoutTab(TXTextControl.Table tbl)
      {
         TXTextControl.TableCell cell = tbl.Cells.GetItem();

         EnableSplitAndMergeBtns(tbl);
         EnableTblCellAlignBtns(cell != null);
         CheckTblCellAlignBtns(cell);
         EnableInsertBtns(tbl);
      }

      private void EnableInsertBtns(TXTextControl.Table tbl)
      {
         if (tbl == null) return;

         _btnTblInsertAbove.Enabled =
         _btnTblInsertBelow.Enabled = tbl.Rows.CanAdd;

         _btnTblInsertLeft.Enabled =
         _btnTblInsertRight.Enabled = tbl.Columns.CanAdd;
      }

      private void CheckTblCellAlignBtns(TXTextControl.TableCell cell)
      {
         // Select correct table cell alignment button

         if (cell != null)
         {
            try
            {
               switch (cell.CellFormat.VerticalAlignment)
               {
                  case TXTextControl.VerticalAlignment.Top:
                     switch (_textControl.Selection.ParagraphFormat.Alignment)
                     {
                        case TXTextControl.HorizontalAlignment.Left:
                           SelectTblCellAlignButton(TblCellAlignButton.TopLeft);
                           break;

                        case TXTextControl.HorizontalAlignment.Center:
                           SelectTblCellAlignButton(TblCellAlignButton.TopCenter);
                           break;

                        case TXTextControl.HorizontalAlignment.Right:
                           SelectTblCellAlignButton(TblCellAlignButton.TopRight);
                           break;

                        case TXTextControl.HorizontalAlignment.Justify:
                           SelectTblCellAlignButton(TblCellAlignButton.TopJust);
                           break;
                     }
                     break;

                  case TXTextControl.VerticalAlignment.Center:
                     switch (_textControl.Selection.ParagraphFormat.Alignment)
                     {
                        case TXTextControl.HorizontalAlignment.Left:
                           SelectTblCellAlignButton(TblCellAlignButton.MidLeft);
                           break;

                        case TXTextControl.HorizontalAlignment.Center:
                           SelectTblCellAlignButton(TblCellAlignButton.MidCenter);
                           break;

                        case TXTextControl.HorizontalAlignment.Right:
                           SelectTblCellAlignButton(TblCellAlignButton.MidRight);
                           break;

                        case TXTextControl.HorizontalAlignment.Justify:
                           SelectTblCellAlignButton(TblCellAlignButton.MidJust);
                           break;
                     }
                     break;

                  case TXTextControl.VerticalAlignment.Bottom:
                     switch (_textControl.Selection.ParagraphFormat.Alignment)
                     {
                        case TXTextControl.HorizontalAlignment.Left:
                           SelectTblCellAlignButton(TblCellAlignButton.BotLeft);
                           break;

                        case TXTextControl.HorizontalAlignment.Center:
                           SelectTblCellAlignButton(TblCellAlignButton.BotCenter);
                           break;

                        case TXTextControl.HorizontalAlignment.Right:
                           SelectTblCellAlignButton(TblCellAlignButton.BotRight);
                           break;

                        case TXTextControl.HorizontalAlignment.Justify:
                           SelectTblCellAlignButton(TblCellAlignButton.BotJust);
                           break;
                     }
                     break;
               } // switch
            }
            catch (Exception exc)
            {
               MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
         }
      }

      private void EnableTblCellAlignBtns(bool bCellExists)
      {
         foreach (RibbonToggleButton btn in _tglBtnTblCellAlign)
         {
            btn.Enabled = bCellExists;
         }
      }

      private void EnableSplitAndMergeBtns(TXTextControl.Table tbl)
      {
         if (tbl == null) return;

         _btnTblMergeCells.Enabled = tbl.CanMergeCells;
         _btnTblSplitCells.Enabled = tbl.CanSplitCells;
      }

      private void TblCellAlignBtnPressed(TblCellAlignButton btn)
      {
         UntoggleToggleButtons(_tglBtnTblCellAlign, _tglBtnTblCellAlign[(int)btn]);

         TXTextControl.Table tbl;
         TXTextControl.ParagraphFormat pf;

         tbl = _textControl.Tables.GetItem();
         if (tbl == null) return;

         TXTextControl.TableCell cell = tbl.Cells.GetItem();
         if (cell == null) return;

         if (cell.Length > 0) _textControl.Select(cell.Start - 1, cell.Length);
         pf = _textControl.Selection.ParagraphFormat;

         switch (btn)
         {
            case TblCellAlignButton.TopLeft:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Top;
               pf.Alignment = TXTextControl.HorizontalAlignment.Left;
               break;

            case TblCellAlignButton.TopCenter:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Top;
               pf.Alignment = TXTextControl.HorizontalAlignment.Center;
               break;

            case TblCellAlignButton.TopRight:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Top;
               pf.Alignment = TXTextControl.HorizontalAlignment.Right;
               break;

            case TblCellAlignButton.TopJust:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Top;
               pf.Alignment = TXTextControl.HorizontalAlignment.Justify;
               break;

            case TblCellAlignButton.MidLeft:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Center;
               pf.Alignment = TXTextControl.HorizontalAlignment.Left;
               break;

            case TblCellAlignButton.MidCenter:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Center;
               pf.Alignment = TXTextControl.HorizontalAlignment.Center;
               break;

            case TblCellAlignButton.MidRight:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Center;
               pf.Alignment = TXTextControl.HorizontalAlignment.Right;
               break;

            case TblCellAlignButton.MidJust:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Center;
               pf.Alignment = TXTextControl.HorizontalAlignment.Justify;
               break;

            case TblCellAlignButton.BotLeft:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Bottom;
               pf.Alignment = TXTextControl.HorizontalAlignment.Left;
               break;

            case TblCellAlignButton.BotCenter:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Bottom;
               pf.Alignment = TXTextControl.HorizontalAlignment.Center;
               break;

            case TblCellAlignButton.BotRight:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Bottom;
               pf.Alignment = TXTextControl.HorizontalAlignment.Right;
               break;

            case TblCellAlignButton.BotJust:
               cell.CellFormat.VerticalAlignment = TXTextControl.VerticalAlignment.Bottom;
               pf.Alignment = TXTextControl.HorizontalAlignment.Justify;
               break;
         }

         if (cell.Length > 0) _textControl.Select(cell.Start - 1, 0);
      }

      private void SelectTblCellAlignButton(TblCellAlignButton btn)
      {
         UntoggleToggleButtons(_tglBtnTblCellAlign);
         _tglBtnTblCellAlign[(int)btn].BooleanValue = true;
      }

      private bool CanRemoveTableCells()
      {
         TXTextControl.Table tbl = _textControl.Tables.GetItem();
         if (tbl == null) return false;
         return tbl.Cells.CanRemove;
      }

      private bool CanRemoveTableColumns()
      {
         TXTextControl.Table tbl = _textControl.Tables.GetItem();
         if (tbl == null) return false;
         return tbl.Columns.CanRemove;
      }

      private bool CanRemoveTableRows()
      {
         TXTextControl.Table tbl = _textControl.Tables.GetItem();
         if (tbl == null) return false;
         return tbl.Rows.CanRemove;
      }

      private bool CanSplitTable()
      {
         TXTextControl.Table tbl = _textControl.Tables.GetItem();
         if (tbl == null) return false;
         return tbl.CanSplit;
      }

      #endregion // Helpers
   }
}