﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Tab_Mailings.cs
** description:	This file contains event handlers and helper methods concerning the "Mailings" ribbon tab.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Microsoft.Data.ConnectionUI;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using System.Threading;
using System.Diagnostics;
using System.Reflection;
using System.IO;
using System.Data;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private RibbonSplitButtonGallery _spltBtnGlryInsMergeField;
      private RibbonDropDownGallery _drpDnGlryMailMrgSelTable;
      private RibbonButton _btnSelectDataSource;
      private RibbonDropDownButton _drpDnInsertSpecialField;
      private RibbonButton _btnInsertIFField;
      private RibbonButton _btnInsertIncludeTextField;
      private RibbonButton _btnInsertDateField;
      private RibbonButton _btnDeleteField;
      private RibbonButton _btnDeleteMergeBlock;
      private RibbonButton _btnGoToFirstRecord;
      private RibbonButton _btnGoToPrevRecord;
      private RibbonButton _btnGoToNextRecord;
      private RibbonButton _btnGoToLastRecord;
      private RibbonToggleButton _tglBtnHighlightMergeFields;
      private RibbonToggleButton[] _tglBtnFieldDisplayMode;
      private RibbonToggleButton _tglBtnPreviewFields;
      private RibbonSplitButton _spltBtnFinishAndMerge;
      private RibbonButton _btnMergeIntoCurrent;
      private RibbonDropDownButton _drpDnMergeSaveSingle;
      private RibbonDropDownButton _drpDnMergeIntoIndivDocs;
      private RibbonButton _btnMergePrint;
      private RibbonButton _btnMergeFinishAndMerge;
      private RibbonButton _btnMergeToPDF;
      private RibbonButton _btnMergeToRTF;
      private RibbonButton _btnMergeToHTML;
      private RibbonButton _btnMergeToDOC;
      private RibbonButton _btnMergeToDOCX;
      private RibbonButton _btnMergeToTXT;
      private RibbonButton _btnMergeToHTMLs;
      private RibbonButton _btnMergeToRTFs;
      private RibbonButton _btnMergeToDOCs;
      private RibbonButton _btnMergeToDOCXs;
      private RibbonButton _btnMergeToTXTs;
      private RibbonButton _btnMergeToPDFs;
      private RibbonSplitButtonGallery _spltBtnGlryInsMergeBlock;
      private RibbonButton _btnFieldProps;
      private RibbonButton _btnSelectSampleDataBase;

      #endregion // Private fields

      #region "  Enumerations  "

      enum RibbonMarkupCommand_TabMailings : uint
      {
         cmdTabMailings = 8000,
         cmdGrpMergeField = 8001,
         cmdGrpMergeBlock = 8044,
         cmdSpltBtnGlryInsMergeField = 8003,
         cmdTglBtnHighlightMergeFields = 8004,
         cmdDrpDnInsertSpecialField = 8005,
         cmdBtnInsertIfField = 8006,
         cmdBtnInsertIncludeTextField = 8007,
         cmdBtnInsertDateField = 8008,
         cmdTglBtnShowFieldCodes = 8009,
         cmdTglBtnShowFieldText = 8010,
         cmdGrpMailMergeView = 8011,
         cmdGrpMailMergeDataSource = 8012,
         cmdGrpMailMergePreview = 8013,
         cmdBtnSelectDataSource = 8014,
         cmdTglBtnPreviewFields = 8016,
         cmdBtnGoToFirstRecord = 8017,
         cmdBtnGoToPrevRecord = 8018,
         cmdBtnGoToNextRecord = 8019,
         cmdBtnGoToLastRecord = 8020,
         cmdDrpDnGlryMailMrgSelTable = 8021,
         cmdSpltBtnFinishAndMerge = 8022,
         cmdGrpMailMergeFinish = 8023,
         cmdBtnMergeIntoCurrent = 8024,
         cmdDrpDnMergeSaveSingle = 8025,
         cmdDrpDnMergeIntoIndivDocs = 8026,
         cmdBtnMergePrint = 8027,
         cmdBtnMergeFinishAndMerge = 8028,
         cmdBtnMergeToPDF = 8029,
         cmdBtnMergeToRTFs = 8030,
         cmdBtnMergeToDOCXs = 8031,
         cmdBtnMergeToDOCs = 8032,
         cmdBtnMergeToHTMLs = 8033,
         cmdBtnMergeToTXTs = 8034,
         cmdBtnMergeToPDFs = 8035,
         cmdBtnMergeToRTF = 8036,
         cmdBtnMergeToDOCX = 8037,
         cmdBtnMergeToDOC = 8038,
         cmdBtnMergeToHTML = 8039,
         cmdBtnMergeToTXT = 8040,
         cmdSpltBtnGlryInsertMergeBlock = 8041,
         cmdBtnDeleteField = 8042,
         cmdBtnDeleteMergeBlock = 8043,
         cmdBtnFieldProps = 8045,
         cmdSpltBtnSelectDataSource = 8046,
         cmdBtnSelectSampleDataBase = 8047,
      }

      #endregion // Enumerations

      #region "  Event handlers  "

      void DrpDnGlryMailMrgSelTable_CategoriesReady(object sender, EventArgs e)
      {
         IUICollection categories = _drpDnGlryMailMrgSelTable.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Tables", CategoryID = 1 });
         categories.Add(new GalleryItemPropertySet() { Label = "Views", CategoryID = 2 });
      }

      void BtnGoToLastRecord_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         GoToLastRecord();
      }

      void BtnGoToNextRecord_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         GoToNextRecord();
      }

      void BtnGoToPrevRecord_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         GoToPreviousRecord();
      }

      void BtnGoToFirstRecord_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         GoToFirstRecord();
      }

      void BtnSelectDataSource_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SelectDataSource();
      }

      void BtnSelectSampleDataBase_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var dir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
         SelectXMLDataSource(dir + "\\sample_db.xml");
      }

      void DrpDnGlryMailMrgSelTable_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            RibbonDropDownGallery glry = sender as RibbonDropDownGallery;
            if (glry == null) return;

            object selItm;
            glry.ItemsSource.GetItem(glry.SelectedItem, out selItm);

            GalleryItemPropertySet propSet = selItm as GalleryItemPropertySet;
            if (selItm == null) return;

            ListAvailableMergeFields(propSet.Label);
            UpdateFieldValues();
         }
      }

      void TglBtnShowFieldText_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         ToggleFieldDisplayModeButton(FieldDisplayMode.ShowFieldText);
      }

      void TglBtnShowFieldCodes_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         ToggleFieldDisplayModeButton(FieldDisplayMode.ShowFieldCodes);
      }

      void TglBtnPreviewFields_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Untoggled:
         if (!_tglBtnPreviewFields.BooleanValue)
         {
            // Set Field display mode back to "show field text"
            _tglBtnFieldDisplayMode[(int)FieldDisplayMode.ShowFieldText].BooleanValue = true;
            ToggleFieldDisplayModeButton(FieldDisplayMode.ShowFieldText);
            return;
         }

         // Toggled:
         if (!MergePossible())
         {
            _tglBtnPreviewFields.BooleanValue = false;
            return;
         }

         // Ask user how much rows to merge
         int nMaxRows = FrmMaxRowsPreview.Show();
         if (nMaxRows == 0)
         {
            // Preview canceled by user
            _tglBtnPreviewFields.BooleanValue = false;
            return;
         }

         // Disable field display mode buttons
         Array.ForEach(_tglBtnFieldDisplayMode, btn => btn.BooleanValue = false);

         EnableMailingsButtons(false);

         // Store current document and generate preview
         BufferCurrentDoc();

         // Merge in separate thread
         PreviewMergeResultAsync(nMaxRows);

         _textControl.EditMode = TXTextControl.EditMode.ReadAndSelect;
      }

      void TglBtnHighlightMergeFields_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonToggleButton btn = sender as RibbonToggleButton;
         if (btn == null) return;

         _bHighlightFields = btn.BooleanValue;
         SetDefaultFieldProperties();
      }

      void SpltBtnInsMergeField_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         object selItem;

         if (e.Key == null)      // Split button pressed
         {
            InsertMergeField();
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            RibbonSplitButtonGallery glry = sender as RibbonSplitButtonGallery;
            if (glry == null) return;

            IUICollection itemsSource = glry.ItemsSource;
            itemsSource.GetItem(glry.SelectedItem, out selItem);
            GalleryItemPropertySet propSet = selItem as GalleryItemPropertySet;
            if (propSet == null) return;

            InsertMergeField(propSet.Label);
         }
      }

      void BtnInsertIncludeTextField_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertIncludeTextField();
         UpdateFieldValues();
      }

      void BtnInsertDateField_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertDateField();
         UpdateFieldValues();
      }

      void BtnInsertIFField_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertIfField();
         UpdateFieldValues();
      }

      void BtnFieldProps_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         FieldSettings();
      }

      void BtnDeleteField_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         DeleteField();
      }

      void BtnDeleteMergeBlock_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         DeleteMergeBlock();
      }
      
      void BtnMergePrint_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergePrintAsync();
      }

      void BtnMergeIntoCurrent_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         if (_fileHandler.DocumentDirty && !FileSaveQuestion()) return;
         MergeIntoCurrentDocAsync();
      }

      void BtnMergeToHTMLs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.HTMLFormat);
      }

      void BtnMergeToDOCs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.MSWord);
      }

      void BtnMergeToDOCXs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.WordprocessingML);
      }

      void BtnMergeToRTFs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.RichTextFormat);
      }

      void BtnMergeToTXTs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.PlainText);
      }

      void BtnMergeToPDFs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeToMultipleFilesAsync(TXTextControl.StreamType.AdobePDF);
      }

      void BtnMergeToPDF_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.AdobePDF);
      }

      void BtnMergeToDOC_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.MSWord);
      }

      void BtnMergeToDOCX_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.WordprocessingML);
      }

      void BtnMergeToTXT_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.PlainText);
      }

      void BtnMergeToHTML_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.HTMLFormat);
      }

      void BtnMergeToRTF_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (!MergePossible()) return;
         MergeSaveSingleDocAsync(TXTextControl.StreamType.RichTextFormat);
      }

      void SpltBtnGlryInsMergeBlock_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         object selItem;

         if (e.Key == null)      // Split button pressed
         {
            InsertMergeBlock(string.Empty);
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            RibbonSplitButtonGallery glry = sender as RibbonSplitButtonGallery;
            if (glry == null) return;

            IUICollection itemsSource = glry.ItemsSource;
            itemsSource.GetItem(glry.SelectedItem, out selItem);
            GalleryItemPropertySet propSet = selItem as GalleryItemPropertySet;
            if (propSet == null) return;

            InsertMergeBlock(propSet.Label);
         }
      }

      #endregion

      #region "  Helpers  "

      private void CreateRibbonItems_TabMailings()
      {
         _spltBtnGlryInsMergeField = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdSpltBtnGlryInsMergeField);
         _drpDnGlryMailMrgSelTable = new RibbonDropDownGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdDrpDnGlryMailMrgSelTable);
         _btnSelectDataSource = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnSelectDataSource);
         _drpDnInsertSpecialField = new RibbonDropDownButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdDrpDnInsertSpecialField);
         _btnInsertIFField = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnInsertIfField);
         _btnInsertIncludeTextField = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnInsertIncludeTextField);
         _btnInsertDateField = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnInsertDateField);
         _btnDeleteField = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnDeleteField);
         _btnFieldProps = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabMailings.cmdBtnFieldProps);
         _btnDeleteMergeBlock = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnDeleteMergeBlock);
         _btnGoToFirstRecord = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnGoToFirstRecord);
         _btnGoToPrevRecord = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnGoToPrevRecord);
         _btnGoToNextRecord = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnGoToNextRecord);
         _btnGoToLastRecord = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnGoToLastRecord);
         _tglBtnHighlightMergeFields = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdTglBtnHighlightMergeFields);
         _tglBtnFieldDisplayMode = new RibbonToggleButton[Enum.GetValues(typeof(FieldDisplayMode)).Length];

         _tglBtnFieldDisplayMode[(int)FieldDisplayMode.ShowFieldCodes] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdTglBtnShowFieldCodes);
         _tglBtnFieldDisplayMode[(int)FieldDisplayMode.ShowFieldText] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdTglBtnShowFieldText);

         _tglBtnPreviewFields = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdTglBtnPreviewFields);
         _spltBtnFinishAndMerge = new RibbonSplitButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdSpltBtnFinishAndMerge);
         _btnMergeIntoCurrent = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeIntoCurrent);
         _drpDnMergeSaveSingle = new RibbonDropDownButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdDrpDnMergeSaveSingle);
         _drpDnMergeIntoIndivDocs = new RibbonDropDownButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdDrpDnMergeIntoIndivDocs);
         _btnMergePrint = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergePrint);
         _btnMergeFinishAndMerge = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeFinishAndMerge);
         _btnMergeToRTFs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToRTFs);
         _btnMergeToDOCXs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToDOCXs);
         _btnMergeToDOCs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToDOCs);
         _btnMergeToHTMLs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToHTMLs);
         _btnMergeToTXTs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToTXTs);
         _btnMergeToPDFs = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToPDFs);
         _btnMergeToPDF = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToPDF);
         _btnMergeToRTF = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToRTF);
         _btnMergeToHTML = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToHTML);
         _btnMergeToDOCX = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToDOCX);
         _btnMergeToDOC = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToDOC);
         _btnMergeToTXT = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdBtnMergeToTXT);
         _spltBtnGlryInsMergeBlock = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabMailings.cmdSpltBtnGlryInsertMergeBlock);
         _btnSelectSampleDataBase = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabMailings.cmdBtnSelectSampleDataBase);
      }

      private void AttachRibbonEvents_TabMailings()
      {
         _spltBtnGlryInsMergeField.ExecuteEvent += SpltBtnInsMergeField_ExecuteEvent;
         _drpDnGlryMailMrgSelTable.ExecuteEvent += DrpDnGlryMailMrgSelTable_ExecuteEvent;
         _drpDnGlryMailMrgSelTable.CategoriesReady += DrpDnGlryMailMrgSelTable_CategoriesReady;
         _btnSelectDataSource.ExecuteEvent += BtnSelectDataSource_ExecuteEvent;
         _btnInsertIFField.ExecuteEvent += BtnInsertIFField_ExecuteEvent;
         _btnInsertIncludeTextField.ExecuteEvent += BtnInsertIncludeTextField_ExecuteEvent;
         _btnInsertDateField.ExecuteEvent += BtnInsertDateField_ExecuteEvent;
         _btnDeleteField.ExecuteEvent += BtnDeleteField_ExecuteEvent;
         _btnFieldProps.ExecuteEvent += BtnFieldProps_ExecuteEvent;
         _btnDeleteMergeBlock.ExecuteEvent += BtnDeleteMergeBlock_ExecuteEvent;
         _btnGoToFirstRecord.ExecuteEvent += BtnGoToFirstRecord_ExecuteEvent;
         _btnGoToPrevRecord.ExecuteEvent += BtnGoToPrevRecord_ExecuteEvent;
         _btnGoToNextRecord.ExecuteEvent += BtnGoToNextRecord_ExecuteEvent;
         _btnGoToLastRecord.ExecuteEvent += BtnGoToLastRecord_ExecuteEvent;
         _tglBtnHighlightMergeFields.ExecuteEvent += TglBtnHighlightMergeFields_ExecuteEvent;

         _tglBtnFieldDisplayMode[(int)FieldDisplayMode.ShowFieldText].ExecuteEvent += TglBtnShowFieldText_ExecuteEvent;
         _tglBtnFieldDisplayMode[(int)FieldDisplayMode.ShowFieldCodes].ExecuteEvent += TglBtnShowFieldCodes_ExecuteEvent;

         _tglBtnPreviewFields.ExecuteEvent += TglBtnPreviewFields_ExecuteEvent;
         _btnMergeIntoCurrent.ExecuteEvent += BtnMergeIntoCurrent_ExecuteEvent;
         _btnMergePrint.ExecuteEvent += BtnMergePrint_ExecuteEvent;
         _btnMergeFinishAndMerge.ExecuteEvent += BtnMergeIntoCurrent_ExecuteEvent;
         _btnMergeToRTFs.ExecuteEvent += BtnMergeToRTFs_ExecuteEvent;
         _btnMergeToDOCXs.ExecuteEvent += BtnMergeToDOCXs_ExecuteEvent;
         _btnMergeToDOCs.ExecuteEvent += BtnMergeToDOCs_ExecuteEvent;
         _btnMergeToHTMLs.ExecuteEvent += BtnMergeToHTMLs_ExecuteEvent;
         _btnMergeToTXTs.ExecuteEvent += BtnMergeToTXTs_ExecuteEvent;
         _btnMergeToPDFs.ExecuteEvent += BtnMergeToPDFs_ExecuteEvent;
         _btnMergeToPDF.ExecuteEvent += BtnMergeToPDF_ExecuteEvent;
         _btnMergeToRTF.ExecuteEvent += BtnMergeToRTF_ExecuteEvent;
         _btnMergeToHTML.ExecuteEvent += BtnMergeToHTML_ExecuteEvent;
         _btnMergeToTXT.ExecuteEvent += BtnMergeToTXT_ExecuteEvent;
         _btnMergeToDOCX.ExecuteEvent += BtnMergeToDOCX_ExecuteEvent;
         _btnMergeToDOC.ExecuteEvent += BtnMergeToDOC_ExecuteEvent;
         _spltBtnGlryInsMergeBlock.ExecuteEvent += SpltBtnGlryInsMergeBlock_ExecuteEvent;
         _btnSelectSampleDataBase.ExecuteEvent += BtnSelectSampleDataBase_ExecuteEvent;
      }

      private void ListAvailableMergeFields(string strTable)
      {
         IUICollection itemsSource = _spltBtnGlryInsMergeField.ItemsSource;
         itemsSource.Clear();
         _dbCon.SelectTable(strTable);
         if (_dbCon.SelectedTable == null) return;

         foreach (string strColName in _dbCon.SelectedTable.ColumnNames)
         {
            itemsSource.Add(new GalleryItemPropertySet()
            {
               Label = strColName,
            });
         }
      }

      private void ListAvailableMergeBlocks()
      {
         IUICollection itemsSource = _spltBtnGlryInsMergeBlock.ItemsSource;
         itemsSource.Clear();

         // Combine table and view names
         List<string> mergeBlockNames = new List<string>();
         mergeBlockNames.AddRange(_dbCon.TableNames);
         mergeBlockNames.AddRange(_dbCon.ViewNames);

         foreach (string strBlockName in mergeBlockNames)
         {
            itemsSource.Add(new GalleryItemPropertySet
            {
               Label = strBlockName,
            });
         }
      }

      private void ToggleFieldDisplayModeButton(FieldDisplayMode dispMode)
      {
         RibbonToggleButton btn = _tglBtnFieldDisplayMode[(int)dispMode];
         if (!btn.BooleanValue)
         {
            // Don't allow button to be untoggled by clicking it
            btn.BooleanValue = true;
            UntoggleToggleButtons(_tglBtnFieldDisplayMode, btn);
            _fldDispModeCur = dispMode;
            return;
         }

         UntoggleToggleButtons(_tglBtnFieldDisplayMode, btn);
         _tglBtnPreviewFields.BooleanValue = false;
         _fldDispModeCur = dispMode;

         EnableMailingsButtons(true);
         ReloadBufferedDoc();

         // Make text control read only when previewing a merge result
         _textControl.EditMode = TXTextControl.EditMode.Edit;

         UpdateFieldValues();
      }

      private void EnableMailingsButtons(bool bEnable)
      {
         _btnSelectDataSource.Enabled =
         _drpDnGlryMailMrgSelTable.Enabled =
         _spltBtnGlryInsMergeField.Enabled =
         _drpDnInsertSpecialField.Enabled =
         _btnDeleteField.Enabled =
         _spltBtnGlryInsMergeBlock.Enabled =
         _btnDeleteMergeBlock.Enabled =
         _btnFieldProps.Enabled =
         _spltBtnFinishAndMerge.Enabled = 
         _tglBtnHighlightMergeFields.Enabled = bEnable;
      }

      void ToggleFieldDisplayButtons()
      {
         _tglBtnPreviewFields.BooleanValue = false;
         _tglBtnFieldDisplayMode[(int)_fldDispModeCur].BooleanValue = true;
      }

      private void SelectDataSource()
      {
         // Show data connection dialog:

         var dcd = new DataConnectionDialog();
         var dcc = new DataConnectionConfiguration();
         dcc.LoadConfiguration(dcd);

         if (DataConnectionDialog.Show(dcd) == System.Windows.Forms.DialogResult.OK)
         {
            _dbCon = new DataProviderConnectionAdapter(dcd.SelectedDataProvider.Name, dcd.ConnectionString);
            RefreshGUIAfterSelectingDB();
         }
      }
      
      private void SelectXMLDataSource(string fileName)
      {
         try
         {
            _dbCon = new XMLFileConnectionAdapter(fileName);
            RefreshGUIAfterSelectingDB();

            // Select default sample db table and list column names
            ListAvailableMergeFields("general");

            MessageBox.Show("Sample database loaded successfully.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      private void RefreshGUIAfterSelectingDB()
      {
         // List available tables and views
         ListTablesAndViews();

         // List available merge blocks
         ListAvailableMergeBlocks();

         // Clear list of available merge fields
         _spltBtnGlryInsMergeField.ItemsSource.Clear();

         UpdateFieldValues();
      }

      #endregion // Helpers
   }
}