﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Tab_View.cs
** description:	This file contains event handlers, fields and helper methods concerning the “View” 
**                ribbon tab.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Windows.Forms;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private RibbonButton _btnPrintLayout;
      private RibbonButton _btnDraft;
      private RibbonCheckBox[] _chkZoom;
      private RibbonCheckBox _chkRulerHor;
      private RibbonCheckBox _chkRulerVert;
      private RibbonCheckBox _chkStatusBar;

      #endregion

      #region "  Enumerations  "

      enum RibbonMarkupCommand_TabView : uint
      {
         cmdTabView = 4000,
         cmdBtnPrintLayout = 4020,
         cmdBtnDraft = 4021,
         cmdDrpDnBtnZoom = 4022,
         cmdChkZoom_025 = 4023,
         cmdChkZoom_050 = 4024,
         cmdChkZoom_075 = 4025,
         cmdChkZoom_100 = 4026,
         cmdChkZoom_150 = 4027,
         cmdChkZoom_200 = 4028,
         cmdChkZoom_400 = 4029,
         cmdChkRulerHor = 4030,
         cmdChkRulerVert = 4031,
         cmdChkStatusBar = 4032,

      }

      enum ZoomButton
      {
         _025,
         _050,
         _075,
         _100,
         _150,
         _200,
         _400,
      }

      #endregion // Enumerations

      #region "  Ribbon event handlers  "

      void ChkStatusBar_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonCheckBox cb = sender as RibbonCheckBox;
         if (cb == null) return;

         _statusBar.Visible = cb.BooleanValue;
      }

      void ChkRulerVert_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonCheckBox cb = sender as RibbonCheckBox;
         if (cb == null) return;

         _rulerBarVert.Visible = cb.BooleanValue;
      }

      void ChkRulerHor_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonCheckBox cb = sender as RibbonCheckBox;
         if (cb == null) return;

         _rulerBarHor.Visible = cb.BooleanValue;
      }

      void ChkZoom_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonCheckBox chkZoom = sender as RibbonCheckBox;
         if (chkZoom == null) return;

         var cmd = (RibbonMarkupCommand_TabView)chkZoom.CommandID;

         if (chkZoom.BooleanValue)
         {
            UntoggleCheckboxes(_chkZoom, chkZoom);

            switch (cmd)
            {
               case RibbonMarkupCommand_TabView.cmdChkZoom_025:
                  _textControl.ZoomFactor = 25;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_050:
                  _textControl.ZoomFactor = 50;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_075:
                  _textControl.ZoomFactor = 75;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_100:
                  _textControl.ZoomFactor = 100;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_150:
                  _textControl.ZoomFactor = 150;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_200:
                  _textControl.ZoomFactor = 200;
                  break;

               case RibbonMarkupCommand_TabView.cmdChkZoom_400:
                  _textControl.ZoomFactor = 400;
                  break;
            }
         }
         else
         {
            chkZoom.BooleanValue = true;  // Dont't allow button to be untoggled
         }
      }

      private void UpdateZoomCheckBoxes()
      {
         // Uncheck all zoom menu items
         foreach (RibbonCheckBox cb in _chkZoom) cb.BooleanValue = false;

         switch (_textControl.ZoomFactor)
         {
            case 25:
               _chkZoom[(int)ZoomButton._025].BooleanValue = true;
               break;
               
            case 50:
               _chkZoom[(int)ZoomButton._050].BooleanValue = true;
               break;

            case 75:
               _chkZoom[(int)ZoomButton._075].BooleanValue = true;
               break;

            case 100:
               _chkZoom[(int)ZoomButton._100].BooleanValue = true;
               break;

            case 150:
               _chkZoom[(int)ZoomButton._150].BooleanValue = true;
               break;

            case 200:
               _chkZoom[(int)ZoomButton._200].BooleanValue = true;
               break;

            case 400:
               _chkZoom[(int)ZoomButton._400].BooleanValue = true;
               break;
         }
      }

      void BtnDraft_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.ViewMode = TXTextControl.ViewMode.Normal;
      }

      void BtnPrintLayout_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.ViewMode = TXTextControl.ViewMode.PageView;
      }

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void CreateRibbonItems_TabView()
      {
         _btnPrintLayout = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdBtnPrintLayout);
         _btnDraft = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdBtnDraft);

         _chkZoom = new RibbonCheckBox[Enum.GetValues(typeof(ZoomButton)).Length];

         _chkZoom[(int)ZoomButton._025] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_025);
         _chkZoom[(int)ZoomButton._050] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_050);
         _chkZoom[(int)ZoomButton._075] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_075);
         _chkZoom[(int)ZoomButton._100] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_100);
         _chkZoom[(int)ZoomButton._150] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_150);
         _chkZoom[(int)ZoomButton._200] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_200);
         _chkZoom[(int)ZoomButton._400] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkZoom_400);

         _chkRulerHor = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkRulerHor);
         _chkRulerVert = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkRulerVert);
         _chkStatusBar = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabView.cmdChkStatusBar);

      }

      private void AttachRibbonEvents_TabView()
      {
         _btnPrintLayout.ExecuteEvent += BtnPrintLayout_ExecuteEvent;
         _btnDraft.ExecuteEvent += BtnDraft_ExecuteEvent;

         _chkZoom[(int)ZoomButton._025].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._050].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._075].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._100].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._150].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._200].ExecuteEvent += ChkZoom_ExecuteEvent;
         _chkZoom[(int)ZoomButton._400].ExecuteEvent += ChkZoom_ExecuteEvent;

         _chkRulerHor.ExecuteEvent += ChkRulerHor_ExecuteEvent;
         _chkRulerVert.ExecuteEvent += ChkRulerVert_ExecuteEvent;
         _chkStatusBar.ExecuteEvent += ChkStatusBar_ExecuteEvent;

      }

      private void UpdateViewCheckBoxes()
      {
         _chkRulerHor.BooleanValue = _rulerBarHor.Visible;
         _chkRulerVert.BooleanValue = _rulerBarVert.Visible;
         _chkStatusBar.BooleanValue = _statusBar.Visible;
      }

      #endregion // Helpers
   }
}
