﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				DbConnectionAdapter.cs
** description:	This file contains an adapter class wrapping a DbConnection object.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Adapter class for DbConnection objects.
   /// </summary>
   internal class XMLFileConnectionAdapter : DbConnectionAdapter
   {
      private DataSet _ds;   // Data set

      internal XMLFileConnectionAdapter(string strXMLFilePath)
      {
         _ds = new DataSet();
         _ds.ReadXml(strXMLFilePath, XmlReadMode.Auto);
         GetTableAndViewNames();
      }

      internal override DataTable GetTable(string strTable)
      {
         return _ds.Tables[strTable];
      }

      internal override void SelectTable(string strTableName) 
      {
         if (_ds == null) return;
         _selectedTable = new DataTableAdapter(GetTable(strTableName));
      }

      internal override void Close()
      {
         _ds = null;
      }

      internal override void GetTableAndViewNames()
      {
         _tableNames = new List<string>();
         _viewNames = new List<string>();

         if (_ds == null) return;

         foreach (DataTable tbl in _ds.Tables)
         {
            TableNames.Add(tbl.TableName);
         }
      }

      /// <summary>
      /// Returns a list of tables from the database.
      /// </summary>
      /// <param name="tableNames">Table names.</param>
      /// <returns>Tables.</returns>
      internal override DataSet GetDataSet(List<string> tableNames)
      {
         // Ignores tableNames because _ds is a dataset already containing
         // all tables in question. The signature of this method
         // is this way to allow the file Mailings.cs to be equal in
         // WPF and WinForms.
         return _ds;
      }

      internal override List<DataRelation> DataRelations
      {
         get 
         { 
            var result = new List<DataRelation>();

            if (_ds != null)
            {
               foreach (DataRelation rel in _ds.Relations)
               {
                  result.Add(rel);
               }
            }

            return result;
         }
      }

      internal override bool AddDataRelation(string mainTableName, string mainColName, string childTableName, string childColName)
      {
         // Fetch tables if necessary
         if (!_ds.Tables.Contains(mainTableName)) return false;
         if (!_ds.Tables.Contains(childTableName)) return false;

         // Create and add relation
         try
         {
            var relation = new DataRelation(
               Guid.NewGuid().ToString(),
               _ds.Tables[mainTableName].Columns[mainColName],
               _ds.Tables[childTableName].Columns[childColName]
            );
            _ds.Relations.Add(relation);
         }
         catch (Exception exc)
         {
            MessageBox.Show("Data relation could not be created: " + exc.Message, "Data Relation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return false;
         }

         return true;
      }
   } // class XMLFileConnectionAdapter
}
